@@ -128,7 +128,7 @@ fn tokenize_a2ml(input: &str) -> Result<Vec<TokenType>, String> {
     let mut amltokens = Vec::<TokenType>::new();
     let mut remaining = input;
 
-    while remaining.len() > 0 {
+    while !remaining.is_empty() {
         let mut chars = remaining.char_indices();
         let (mut idx, mut c) = chars.next().unwrap();
 
@@ -153,7 +153,7 @@ fn tokenize_a2ml(input: &str) -> Result<Vec<TokenType>, String> {
                     c = pair.1;
                     if c == '*' {
                         star = true;
-                    } else if c == '/' && star == true {
+                    } else if c == '/' && star {
                         done = true;
                     } else {
                         star = false;
@@ -372,7 +372,7 @@ pub(crate) fn parse_a2ml(input: &str) -> Result<A2mlTypeSpec, String> {
     if let Some(ifdata_block) = ifdata_block {
         Ok(ifdata_block)
     } else {
-        Err(format!("The A2ML declaration was fully parsed. However it does not contain an IF_DATA block, so it is not usable."))
+        Err("The A2ML declaration was fully parsed. However it does not contain an IF_DATA block, so it is not usable.".to_string())
     }
 }
 
@@ -393,10 +393,10 @@ fn parse_aml_type(tok_iter: &mut A2mlTokenIter, types: &TypeSet, tok_start: &Tok
         TokenType::Uint64 => Ok((None, A2mlTypeSpec::UInt64)),
         TokenType::Float => Ok((None, A2mlTypeSpec::Float)),
         TokenType::Double => Ok((None, A2mlTypeSpec::Double)),
-        TokenType::Enum => parse_aml_type_enum(tok_iter, &types),
-        TokenType::Struct => parse_aml_type_struct(tok_iter, &types),
-        TokenType::Taggedstruct => parse_aml_type_taggedstruct(tok_iter, &types),
-        TokenType::Taggedunion => parse_aml_type_taggedunion(tok_iter, &types),
+        TokenType::Enum => parse_aml_type_enum(tok_iter, types),
+        TokenType::Struct => parse_aml_type_struct(tok_iter, types),
+        TokenType::Taggedstruct => parse_aml_type_taggedstruct(tok_iter, types),
+        TokenType::Taggedunion => parse_aml_type_taggedunion(tok_iter, types),
         _ => Err(format!("unexpected token {:?} in type declaration", tok_start)),
     }
 }
@@ -416,8 +416,7 @@ fn parse_aml_type_enum(tok_iter: &mut A2mlTokenIter, types: &TypeSet) -> Result<
     // check if this is a reference to a previous declaration or if there is also a list of items in {}
     let tok_peek = tok_iter.peek();
     if tok_peek.is_none() || **tok_peek.unwrap() != TokenType::OpenCurlyBracket {
-        if name.is_some() {
-            let name = String::from(name.unwrap());
+        if let Some(name) = name {
             if let Some(A2mlTypeSpec::Enum(items)) = types.enums.get(&name) {
                 return Ok( (Some(name), A2mlTypeSpec::Enum(items.clone())) );
             } else {
@@ -468,8 +467,7 @@ fn parse_aml_type_struct(tok_iter: &mut A2mlTokenIter, types: &TypeSet) -> Resul
     // check if this is a reference to a previous declaration or if there is also a definition of the struct enclosed in {}
     let tok_peek = tok_iter.peek();
     if tok_peek.is_none() || **tok_peek.unwrap() != TokenType::OpenCurlyBracket {
-        if name.is_some() {
-            let name = String::from(name.unwrap());
+        if let Some(name) = name {
             if let Some(A2mlTypeSpec::Struct(structitems)) = types.structs.get(&name) {
                 return Ok( (Some(name), A2mlTypeSpec::Struct(structitems.clone())) );
             } else {
@@ -485,7 +483,7 @@ fn parse_aml_type_struct(tok_iter: &mut A2mlTokenIter, types: &TypeSet) -> Resul
     let mut structdata = Vec::new();
 
     loop {
-        structdata.push(parse_aml_member(tok_iter, &types)?);
+        structdata.push(parse_aml_member(tok_iter, types)?);
         require_token_type(tok_iter, TokenType::Semicolon)?;
 
         if let Some(TokenType::ClosedCurlyBracket) = tok_iter.peek() {
@@ -508,8 +506,7 @@ fn parse_aml_type_taggedstruct(tok_iter: &mut A2mlTokenIter, types: &TypeSet) ->
     // check if this is a reference to a previous declaration or if there is also a definition of the taggedstruct enclosed in {}
     let tok_peek = tok_iter.peek();
     if tok_peek.is_none() || **tok_peek.unwrap() != TokenType::OpenCurlyBracket {
-        if name.is_some() {
-            let name = String::from(name.unwrap());
+        if let Some(name) = name {
             if let Some(A2mlTypeSpec::TaggedStruct(tsitems)) = types.taggedstructs.get(&name) {
                 return Ok( (Some(name), A2mlTypeSpec::TaggedStruct(tsitems.clone())) );
             } else {
@@ -524,7 +521,7 @@ fn parse_aml_type_taggedstruct(tok_iter: &mut A2mlTokenIter, types: &TypeSet) ->
     require_token_type(tok_iter, TokenType::OpenCurlyBracket)?; // guaranteed to succeed
     let mut taggedstructdata = HashMap::new();
     loop {
-        let (itemname, itemdef) = parse_aml_taggedmember(tok_iter, &types, true)?;
+        let (itemname, itemdef) = parse_aml_taggedmember(tok_iter, types, true)?;
         taggedstructdata.insert(itemname, itemdef);
         require_token_type(tok_iter, TokenType::Semicolon)?;
 
@@ -547,8 +544,7 @@ fn parse_aml_type_taggedunion(tok_iter: &mut A2mlTokenIter, types: &TypeSet) ->
     /* check if this is a reference to a previous declaration or if there is also a definition of the taggedunion enclosed in {} */
     let tok_peek = tok_iter.peek();
     if tok_peek.is_none() || **tok_peek.unwrap() != TokenType::OpenCurlyBracket {
-        if name.is_some() {
-            let name = String::from(name.unwrap());
+        if let Some(name) = name {
             if let Some(A2mlTypeSpec::TaggedUnion(tsitems)) = types.taggedunions.get(&name) {
                 return Ok( (Some(name), A2mlTypeSpec::TaggedUnion(tsitems.clone())) );
             } else {
@@ -563,7 +559,7 @@ fn parse_aml_type_taggedunion(tok_iter: &mut A2mlTokenIter, types: &TypeSet) ->
     require_token_type(tok_iter, TokenType::OpenCurlyBracket)?; // guaranteed to succeed
     let mut taggeduniondata = HashMap::new();
     loop {
-        let (itemname, itemdef) = parse_aml_taggedmember(tok_iter, &types, false)?;
+        let (itemname, itemdef) = parse_aml_taggedmember(tok_iter, types, false)?;
         taggeduniondata.insert(itemname, itemdef);
         require_token_type(tok_iter, TokenType::Semicolon)?;
 
@@ -601,7 +597,7 @@ fn parse_aml_taggedmember(tok_iter: &mut A2mlTokenIter, types: &TypeSet, allow_r
             let item = if let Some(TokenType::Semicolon) = tok_peek {
                 A2mlTypeSpec::None
             } else {
-                parse_aml_tagged_def(tok_iter, &types)?
+                parse_aml_tagged_def(tok_iter, types)?
             };
             (tag.to_string(), A2mlTaggedTypeSpec { tag: tag.to_string(), item, is_block, repeat })
         } else {
@@ -627,7 +623,7 @@ fn parse_aml_tagged_def(tok_iter: &mut A2mlTokenIter, types: &TypeSet) -> Result
         tok_iter.next();
     }
 
-    let mut member = parse_aml_member(tok_iter, &types)?;
+    let mut member = parse_aml_member(tok_iter, types)?;
 
     if inner_repeat {
         require_token_type(tok_iter, TokenType::ClosedRoundBracket)?;
@@ -645,7 +641,7 @@ fn parse_aml_tagged_def(tok_iter: &mut A2mlTokenIter, types: &TypeSet) -> Result
 //    array_specifier = "[" constant "]" | "[" constant "]" array_specifier
 fn parse_aml_member(tok_iter: &mut A2mlTokenIter, types: &TypeSet) -> Result<A2mlTypeSpec, String> {
     let tok_start = nexttoken(tok_iter)?;
-    let (_, mut base_type) = parse_aml_type(tok_iter, &types, tok_start)?;
+    let (_, mut base_type) = parse_aml_type(tok_iter, types, tok_start)?;
 
     while let Some(TokenType::OpenSquareBracket) = tok_iter.peek() {
         /* get the array dim */
@@ -717,21 +713,21 @@ fn nexttoken<'a>(tok_iter: &mut A2mlTokenIter<'a>) -> Result<&'a TokenType<'a>,
 
 
 impl GenericIfData {
-    pub fn get_block_items(&self) -> Result<(Option<String>, u32, &Vec<GenericIfData>), ()> {
+    pub fn get_block_items(&self) -> Result<(Option<String>, u32, &Vec<GenericIfData>), &'static str> {
         match self {
             GenericIfData::Block{incfile, line, items} => Ok((incfile.clone(), *line, items)),
-            _ => Err(())
+            _ => Err("strucutal mismatch: get_block_items called on something that is not a Block")
         }
     }
 
-    pub fn get_struct_items(&self) -> Result<(Option<String>, u32, &Vec<GenericIfData>), ()> {
+    pub fn get_struct_items(&self) -> Result<(Option<String>, u32, &Vec<GenericIfData>), &'static str> {
         match self {
             GenericIfData::Struct(file, line, blockitems) => Ok((file.clone(), *line, blockitems)),
-            _ => Err(())
+            _ => Err("strucutal mismatch: get_struct_items called on something that is not a Struct")
         }
     }
 
-    pub fn get_int_is_hex(&self) -> Result<bool, ()> {
+    pub fn get_int_is_hex(&self) -> Result<bool, &'static str> {
         match self {
             GenericIfData::UChar(_, (_, is_hex)) |
             GenericIfData::UInt(_, (_, is_hex)) |
@@ -741,115 +737,115 @@ impl GenericIfData {
             GenericIfData::Int(_, (_, is_hex)) |
             GenericIfData::Long(_, (_, is_hex)) |
             GenericIfData::Int64(_, (_, is_hex)) => Ok(*is_hex),
-            _ => Err(())
+            _ => Err("strucutal mismatch: get_int_is_hex called on something that is not an integer")
         }
     }
 
-    pub fn get_integer_u8(&self) -> Result<u8, ()> {
+    pub fn get_integer_u8(&self) -> Result<u8, &'static str> {
         if let GenericIfData::UChar(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_u8 called on something that is not a UChar")
         }
     }
 
-    pub fn get_integer_u16(&self) -> Result<u16, ()> {
+    pub fn get_integer_u16(&self) -> Result<u16, &'static str> {
         if let GenericIfData::UInt(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_u16 called on something that is not a UInt")
         }
     }
 
-    pub fn get_integer_u32(&self) -> Result<u32, ()> {
+    pub fn get_integer_u32(&self) -> Result<u32, &'static str> {
         if let GenericIfData::ULong(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_u32 called on something that is not a ULong")
         }
     }
 
-    pub fn get_integer_u64(&self) -> Result<u64, ()> {
+    pub fn get_integer_u64(&self) -> Result<u64, &'static str> {
         if let GenericIfData::UInt64(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_u64 called on something that is not a UInt64")
         }
     }
 
-    pub fn get_integer_i8(&self) -> Result<i8, ()> {
+    pub fn get_integer_i8(&self) -> Result<i8, &'static str> {
         if let GenericIfData::Char(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_i8 called on something that is not a Char")
         }
     }
 
-    pub fn get_integer_i16(&self) -> Result<i16, ()> {
+    pub fn get_integer_i16(&self) -> Result<i16, &'static str> {
         if let GenericIfData::Int(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_i16 called on something that is not an Int")
         }
     }
 
-    pub fn get_integer_i32(&self) -> Result<i32, ()> {
+    pub fn get_integer_i32(&self) -> Result<i32, &'static str> {
         if let GenericIfData::Long(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_i32 called on something that is not a Long")
         }
     }
 
-    pub fn get_integer_i64(&self) -> Result<i64, ()> {
+    pub fn get_integer_i64(&self) -> Result<i64, &'static str> {
         if let GenericIfData::Int64(_, (val, _)) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_integer_i64 called on something that is not an Int64")
         }
     }
 
-    pub fn get_float(&self) -> Result<f32, ()> {
+    pub fn get_float(&self) -> Result<f32, &'static str> {
         if let GenericIfData::Float(_, val) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_float called on something that is not a Float")
         }
     }
 
-    pub fn get_double(&self) -> Result<f64, ()> {
+    pub fn get_double(&self) -> Result<f64, &'static str> {
         if let GenericIfData::Double(_, val) = self {
             Ok(*val)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_double called on something that is not a Double")
         }
     }
 
-    pub fn get_stringval(&self) -> Result<String, ()> {
+    pub fn get_stringval(&self) -> Result<String, &'static str> {
         if let GenericIfData::String(_, val) = self {
             Ok(val.to_owned())
         } else {
-            Err(())
+            Err("strucutal mismatch: get_stringval called on something that is not a String")
         }
     }
 
-    pub fn get_array(&self) -> Result<&Vec<GenericIfData>, ()> {
+    pub fn get_array(&self) -> Result<&Vec<GenericIfData>, &'static str> {
         if let GenericIfData::Array(arrayitems) = self {
             Ok(arrayitems)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_array called on something that is not an Array")
         }
     }
 
-    pub fn get_sequence(&self) -> Result<&Vec<GenericIfData>, ()> {
+    pub fn get_sequence(&self) -> Result<&Vec<GenericIfData>, &'static str> {
         if let GenericIfData::Sequence(seqitems) = self {
             Ok(seqitems)
         } else {
-            Err(())
+            Err("strucutal mismatch: get_sequence called on something that is not a Sequence")
         }
     }
 
-    pub fn get_line(&self) -> Result<u32, ()> {
+    pub fn get_line(&self) -> Result<u32, &'static str> {
         match self {
             GenericIfData::Char(line, _) |
             GenericIfData::Int(line, _) |
@@ -865,11 +861,11 @@ impl GenericIfData {
             GenericIfData::EnumItem(line, _) |
             GenericIfData::Struct(_, line, _) |
             GenericIfData::Block {line, .. } => Ok(*line),
-            _ => Err(())
+            _ => Err("strucutal mismatch: get_line called on something that has no line info")
         }
     }
 
-    pub fn get_single_optitem<T>(&self, tag: &str, func: fn (&GenericIfData, u32, u32, u32) -> Result<T, ()>) -> Result<Option<T>, ()> {
+    pub fn get_single_optitem<T>(&self, tag: &str, func: fn (&GenericIfData, u32, u32, u32) -> Result<T, &'static str>) -> Result<Option<T>, &'static str> {
         match self {
             GenericIfData::TaggedStruct(taggeditems) |
             GenericIfData::TaggedUnion(taggeditems) => {
@@ -880,12 +876,12 @@ impl GenericIfData {
                 }
             }
             _ => {
-                Err(())
+                Err("strucutal mismatch: get_single_optitem called on unsuitable element")
             }
         }
     }
 
-    pub fn get_multiple_optitems<T>(&self, tag: &str, func: fn (&GenericIfData, u32, u32, u32) -> Result<T, ()>) -> Result<Vec<T>, ()> {
+    pub fn get_multiple_optitems<T>(&self, tag: &str, func: fn (&GenericIfData, u32, u32, u32) -> Result<T, &'static str>) -> Result<Vec<T>, &'static str> {
         match self {
             GenericIfData::TaggedStruct(taggeditems) |
             GenericIfData::TaggedUnion(taggeditems) => {
@@ -898,7 +894,7 @@ impl GenericIfData {
                 Ok(resultvec)
             }
             _ => {
-                Err(())
+                Err("strucutal mismatch: get_multiple_optitems called on unsuitable element")
             }
         }
     }
@@ -972,7 +968,7 @@ impl GenericIfData {
             Self::TaggedStruct(taggeditems) |
             Self::TaggedUnion(taggeditems) => {
                 let mut tgroup = Vec::new();
-                for (_, tgitemlist) in taggeditems {
+                for tgitemlist in taggeditems.values() {
                     for tgitem in tgitemlist {
                         tgroup.push(TaggedItemInfo::build_generic(tgitem.data.write(indent + 1), tgitem));
                     }
@@ -1001,7 +997,7 @@ impl GenericIfData {
             }
             Self::TaggedStruct(taggeditems) |
             Self::TaggedUnion(taggeditems) => {
-                for (_, tgitemlist) in taggeditems {
+                for tgitemlist in taggeditems.values_mut() {
                     for tgitem in tgitemlist {
                         tgitem.incfile = None;
                         tgitem.data.merge_includes();
@@ -1020,7 +1016,7 @@ impl PartialEq for GenericIfData {
     fn eq(&self, other: &Self) -> bool {
         match self {
             Self::None => {
-                if let Self::None = other { true } else { false }
+                matches!(other, Self::None)
             }
             Self::Char(_, val) => {
                 if let Self::Char(_, otherval) = other { val == otherval } else { false }
