@@ -35,18 +35,17 @@ pub(crate) struct TokenResult {
 // possible to do this because characters outside of basic ASCII can actually only occur in
 // strings and comments. UTF-8 in strings is directly copied to the output, while comments are discarded.
 // An important extra goal of the tokenizer is to attach the source line number to each token so that error messages can give accurate location info
-pub(crate) fn tokenize(filename: String, fileid: usize, filedata: &str) -> Result<TokenResult, String> {
+pub(crate) fn tokenize(filename: String, fileid: usize, filetext: &str) -> Result<TokenResult, String> {
     let mut filenames: Vec<String> = vec![filename];
-    let filebytes = filedata.as_bytes();
-    let mut tokens: Vec<A2lToken> = Vec::with_capacity(filedata.len() / 20);
+    let mut filedatas: Vec<String> = vec![filetext.to_owned()];
+    let filebytes = filetext.as_bytes();
+    let datalen = filebytes.len();
+
+    let mut tokens: Vec<A2lToken> = Vec::with_capacity(datalen / 20);
     let mut next_fileid = fileid + 1;
-    let datalen = filedata.len();
     let mut bytepos = 0;
     let mut separated = true;
     let mut line = 1;
-    let mut filedatas: Vec<String> = vec![];
-
-    filedatas.push(filedata.to_string());
 
     while bytepos < datalen {
         let startpos = bytepos;
@@ -72,17 +71,17 @@ pub(crate) fn tokenize(filename: String, fileid: usize, filedata: &str) -> Resul
                 while bytepos < datalen && filebytes[bytepos] != b'\n' {
                     bytepos += 1;
                 }
-            } else if filedata[bytepos .. ].starts_with("begin") {
+            } else if filebytes[bytepos .. ].starts_with(b"begin") {
                 separator_check(separated, line)?;
                 bytepos += 5;
                 tokens.push(A2lToken{ttype: A2lTokenType::Begin, startpos, endpos: bytepos, fileid, line});
                 separated = false;
-            } else if filedata[bytepos .. ].starts_with("end") {
+            } else if filebytes[bytepos .. ].starts_with(b"end") {
                 separator_check(separated, line)?;
                 bytepos += 3;
                 tokens.push(A2lToken{ttype: A2lTokenType::End, startpos, endpos: bytepos, fileid, line});
                 separated = false;
-            } else if filedata[bytepos .. ].starts_with("include") {
+            } else if filebytes[bytepos .. ].starts_with(b"include") {
                 separator_check(separated, line)?;
                 bytepos += 7;
                 tokens.push(A2lToken{ttype: A2lTokenType::Include, startpos, endpos: bytepos, fileid, line});
@@ -107,7 +106,7 @@ pub(crate) fn tokenize(filename: String, fileid: usize, filedata: &str) -> Resul
             tokens.push(A2lToken{ttype: A2lTokenType::Identifier, startpos, endpos: bytepos, fileid, line});
             separated = false;
 
-            let (new_bytepos, new_line) = handle_a2ml(filedata, bytepos, line, fileid, &mut tokens);
+            let (new_bytepos, new_line) = handle_a2ml(filetext, bytepos, line, fileid, &mut tokens);
             if bytepos != new_bytepos {
                 separated = true;
             }
@@ -120,10 +119,10 @@ pub(crate) fn tokenize(filename: String, fileid: usize, filedata: &str) -> Resul
             while bytepos < datalen && is_numchar(filebytes[bytepos]) {
                 bytepos += 1;
             }
-            let number = &filedata[startpos..bytepos];
-            if number == "-" {
+            let number = &filebytes[startpos..bytepos];
+            if number == b"-" {
                 return Err(format!("Error: Invalid numerical constant consisting of only \"-\" found on line {}", line));
-            } else if number == "0x" {
+            } else if number == b"0x" {
                 return Err(format!("Error: Invalid numerical constant consisting of only \"0x\" found on line {}", line));
             }
             tokens.push(A2lToken{ttype: A2lTokenType::Number, startpos, endpos: bytepos, fileid, line});
@@ -146,14 +145,14 @@ pub(crate) fn tokenize(filename: String, fileid: usize, filedata: &str) -> Resul
                     filename_start += 1;
                     filename_end -= 1;
                 }
-                let incname = &filedata[filename_start .. filename_end];
+                let incname = &filetext[filename_start .. filename_end];
 
-                let incfilename = make_include_filename(&incname, &filenames[0]);
+                let incfilename = make_include_filename(incname, &filenames[0]);
 
                 // check if incname is an accessible file
                 let loadresult = loader::load(&incfilename);
                 if let Ok(incfiledata) = loadresult {
-                    let mut tokresult = tokenize(incname.to_string(), next_fileid, &incfiledata)?;
+                    let mut tokresult = tokenize(incname.to_owned(), next_fileid, &incfiledata)?;
 
                     next_fileid += tokresult.filenames.len();
 
@@ -252,35 +251,68 @@ fn find_string_end(filebytes: &[u8],  mut bytepos: usize, line: u32) -> Result<u
 // handle_a2ml finds the end of the A2ML block and stores its content as a string
 fn handle_a2ml(filedata: &str, mut bytepos: usize, mut line: u32, fileid: usize, tokens: &mut Vec<A2lToken>) -> (usize, u32) {
     let tokcount = tokens.len();
-    if tokcount >= 2 {
-        if tokens[tokcount-2].ttype == A2lTokenType::Begin {
-            let startpos = bytepos;
-            let filebytes = filedata.as_bytes();
-            let datalen = filedata.len();
-            let tag = &filedata[tokens[tokcount-1].startpos .. tokens[tokcount-1].endpos];
+    if tokcount >= 2 && tokens[tokcount-2].ttype == A2lTokenType::Begin {
+        let startpos = bytepos;
+        let filebytes = filedata.as_bytes();
+        let datalen = filedata.len();
+        let tag = &filedata[tokens[tokcount-1].startpos .. tokens[tokcount-1].endpos];
 
-            if tag == "A2ML" {
-                while bytepos < datalen && !(filebytes[bytepos] == b'/' && filedata[bytepos .. ].starts_with("/end A2ML")) {
+        if tag == "A2ML" {
+            let mut done = false;
+            while !done && bytepos < datalen {
+                // find the next occurrence of '/'
+                // this should be the start of one of "/*", "//", or "/end"
+                while bytepos < datalen && filebytes[bytepos] != b'/' {
                     bytepos += 1;
                 }
-
-                // trim off trailing whitespace up to and including the last newline - this newline and the
-                // following indentation will be written together with /end A2ML
-                while filebytes[bytepos-1].is_ascii_whitespace() && filebytes[bytepos-1] != b'\r' && filebytes[bytepos-1] != b'\n' {
-                    bytepos -= 1;
-                }
-                if filebytes[bytepos-1] == b'\r' && filebytes[bytepos-1] == b'\n' {
-                    bytepos -= 2;
-                } else if filebytes[bytepos-1] == b'\n' {
-                    bytepos -= 1;
+                if filebytes[bytepos ..].starts_with(b"//") {
+                    // line comment
+                    // skip the comment marker
+                    bytepos += 2;
+                    // skip the remaining characters on the line up to the newline
+                    while bytepos < datalen && filebytes[bytepos] != b'\n' {
+                        bytepos += 1;
+                    }
+                } else if filebytes[bytepos ..].starts_with(b"/*") {
+                    // block comment
+                    // skip the comment marker
+                    bytepos += 2;
+                    // skip the remaining characters on the line up to the newline
+                    while bytepos < (datalen - 1) && !(filebytes[bytepos] == b'*' && filebytes[bytepos + 1] == b'/') {
+                        bytepos += 1;
+                    }
+                    bytepos += 2;
+                    if bytepos > datalen {
+                        bytepos = datalen;
+                    }
+                } else if filebytes[bytepos ..].starts_with(b"/end") {
+                    done = true;
+                } else {
+                    // solitary '/' hanging around? this will definitely be a parse error later on
+                    bytepos += 1;
                 }
             }
 
-            if bytepos > startpos {
-                tokens.push(A2lToken{ttype: A2lTokenType::String, startpos, endpos: bytepos, fileid, line});
+            // while bytepos < datalen && !(filebytes[bytepos] == b'/' && filedata[bytepos .. ].starts_with("/end A2ML")) {
+            //     bytepos += 1;
+            // }
 
-                line += count_newlines(&filebytes[startpos .. bytepos]);
+            // trim off trailing whitespace up to and including the last newline - this newline and the
+            // following indentation will be written together with /end A2ML
+            while filebytes[bytepos-1].is_ascii_whitespace() && filebytes[bytepos-1] != b'\r' && filebytes[bytepos-1] != b'\n' {
+                bytepos -= 1;
             }
+            if filebytes[bytepos-1] == b'\r' && filebytes[bytepos-1] == b'\n' {
+                bytepos -= 2;
+            } else if filebytes[bytepos-1] == b'\n' {
+                bytepos -= 1;
+            }
+        }
+
+        if bytepos > startpos {
+            tokens.push(A2lToken{ttype: A2lTokenType::String, startpos, endpos: bytepos, fileid, line});
+
+            line += count_newlines(&filebytes[startpos .. bytepos]);
         }
     }
 
@@ -457,3 +489,26 @@ fn tokenize_a2l_skip_whitespace() {
     let tokresult = tokenize("testcase".to_string(), 0, &data).expect("Error");
     assert_eq!(tokresult.tokens.len(), 0);
 }
+
+#[test]
+fn tokenize_skip_a2ml() {
+    let data = String::from(r##"
+ASAP2_VERSION 1 60
+/begin PROJECT Test "test test"
+    /begin MODULE MODULE_NAME ""
+        /begin A2ML
+            struct Foo {
+                uint;
+                uint;
+            }; /* trap: /end A2ML */
+            / / //
+        /end A2ML
+    /end MODULE
+/end PROJECT
+"##);
+    let tokresult = tokenize("testcase".to_string(), 0, &data).expect("Error");
+    println!("token count: {}", tokresult.tokens.len());
+    assert_eq!(tokresult.tokens.len(), 20);
+    assert_eq!(tokresult.tokens[0].ttype, A2lTokenType::Identifier);
+    assert_eq!(tokresult.tokens[13].ttype, A2lTokenType::String); // a2ml body text
+}
\ No newline at end of file
