@@ -17,20 +17,11 @@ ASAP2_VERSION 1 61
   /end MODULE
 /end PROJECT"###;
 
-    struct A2lLogger {
-        log: Vec<String>
-    }
-    
-    impl a2lfile::Logger for A2lLogger {
-        fn log_message(&mut self, msg: String) {
-            self.log.push(msg);
-        }
-    }
 
     #[test]
     fn round_trip() {
-        let mut logger = A2lLogger { log: Vec::new() };
-        let a2lfile = a2lfile::load_from_string(TEST_A2L, None, &mut logger, false).unwrap();
+        let mut log_msgs = Vec::<String>::new();
+        let a2lfile = a2lfile::load_from_string(TEST_A2L, None, &mut log_msgs, false).unwrap();
         let text = a2lfile.write_to_string();
         println!("input:\n{}\noutput:\n{}\n", TEST_A2L, text);
 
